@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,6 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <stdio.h>
-#include <stddef.h>
-#include <stdint.h>
-
 #include "regionalloctracking.h"
 
 #ifndef DOSBOX_BIOS_H
@@ -168,7 +164,7 @@ Bitu ROMBIOS_GetMemory(Bitu bytes,const char *who=NULL,Bitu alignment=1,Bitu mus
 extern RegionAllocTracking rombios_alloc;
 
 /* maximum of scancodes handled by keyboard bios routines */
-#define MAX_SCAN_CODE 0x58
+#define MAX_SCAN_CODE 0x7F
 
 /* The Section handling Bios Disk Access */
 //#define BIOS_MAX_DISK 10
@@ -329,6 +325,19 @@ public:
     size_t              alloc_sz = 0;
 };
 
+extern bool ACPI_enabled;
+extern uint32_t ACPI_BASE;
+extern uint32_t ACPI_REGION_SIZE; // power of 2
+extern uint32_t ACPI_version;
+extern unsigned char *ACPI_buffer;
+extern size_t ACPI_buffer_size;
+extern int ACPI_IRQ;
+extern unsigned int ACPI_SMI_CMD;
+
+void ACPI_mem_enable(const bool enable);
+void ACPI_free();
+bool ACPI_init();
+
 /* abc = ASCII letters of the alphabet
  * defg = hexadecimal digits */
 #define ISAPNP_ID(a,b,c,d,e,f,g) \
