@@ -195,19 +195,19 @@ public:
 		SetName("EMMXXXX0");
 		GEMMIS_seg=0;
 	}
-	bool Read(uint8_t * /*data*/,uint16_t * /*size*/) { return false;}
-	bool Write(const uint8_t * /*data*/,uint16_t * /*size*/){ 
+	bool Read(uint8_t * /*data*/,uint16_t * /*size*/) override { return false;}
+	bool Write(const uint8_t * /*data*/,uint16_t * /*size*/) override {
 		LOG(LOG_IOCTL,LOG_NORMAL)("EMS:Write to device");	
 		return false;
 	}
-	bool Seek(uint32_t * /*pos*/,uint32_t /*type*/){return false;}
-	bool Close(){return false;}
-	uint16_t GetInformation(void)
+	bool Seek(uint32_t * /*pos*/,uint32_t /*type*/) override {return false;}
+	bool Close() override {return false;}
+	uint16_t GetInformation(void) override
 	{
 		return DeviceInfoFlags::Device | DeviceInfoFlags::IoctlSupport;
 	}
-	bool ReadFromControlChannel(PhysPt bufptr,uint16_t size,uint16_t * retcode);
-	bool WriteToControlChannel(PhysPt /*bufptr*/,uint16_t /*size*/,uint16_t * /*retcode*/){return true;}
+	bool ReadFromControlChannel(PhysPt bufptr,uint16_t size,uint16_t * retcode) override;
+	bool WriteToControlChannel(PhysPt /*bufptr*/,uint16_t /*size*/,uint16_t * /*retcode*/) override {return true;}
 private:
 //	uint8_t cache;
 	bool is_emm386;
@@ -394,7 +394,7 @@ static uint8_t EMM_AllocateSystemHandle(uint16_t pages/*NTS: EMS pages are 16KB,
 	 * Why? Well, if the system handle exists on an even megabyte, then it no longer matters what state the A20
 	 * gate is in. If the DOS game relies on EMM386.EXE but fiddles with the A20 gate while running in virtual 8086
 	 * mode, setting this option can help avoid crashes, where normally clearing the A20 gate prevents EMM386.EXE
-	 * from using it's protected mode structures (GDT, page tables) and the sudden aliasing turns them into junk
+	 * from using its protected mode structures (GDT, page tables) and the sudden aliasing turns them into junk
 	 * and the system crashes (random contents in DOS conventional memory interpreted as protected mode structures
 	 * doesn't work very well). */
 	mem = 0;
@@ -731,7 +731,7 @@ static uint8_t GetSetHandleName(void) {
 
 static uint8_t GetSetHandleAttributes(void) {
 	switch (reg_al) {
-	case 0x00:	// Get handle attribubtes
+	case 0x00:	// Get handle attributes
 		if (!ValidHandle(reg_dx)) return EMM_INVALID_HANDLE;
 		reg_al = EMM_VOLATILE;	// We only support volatile
 		break;
@@ -1635,7 +1635,7 @@ public:
 			return;
 		}
 
-        LOG_MSG("EMS page frame at 0x%04x-0x%04x",EMM_PAGEFRAME,EMM_PAGEFRAME+0xFFF);
+		LOG_MSG("EMS page frame at 0x%04x-0x%04x",EMM_PAGEFRAME,EMM_PAGEFRAME+0xFFF);
 
 		ENABLE_VCPI = section->Get_bool("vcpi");
 		ENABLE_V86_STARTUP = section->Get_bool("emm386 startup active");
@@ -1648,30 +1648,30 @@ public:
 			ENABLE_VCPI = false;
 		}
 
-        bool XMS_Active(void);
+		bool XMS_Active(void);
 
-        /* if XMS is not enabled, EMM386 emulation is impossible.
-         * Real MS-DOS EMM386.EXE will flat out refuse to load if HIMEM.SYS is not loaded.
-         * that also prevents VCPI from working. */
-        if (!XMS_Active() && ems_type != EMS_BOARD) {
-            if (ems_type == EMS_MIXED) {
-                LOG_MSG("EMS changed to board mode and VCPI disabled, because XMS is not enabled.");
-                ems_type = EMS_BOARD;
-            }
-            else if (ems_type == EMS_EMM386) {
-                /* do as MS-DOS does */
-                setup_EMS_none();
-                ems_type = EMS_NONE;
-                LOG_MSG("EMS disabled, EMM386 emulation is impossible when XMS is not enabled");
-                return;
-            }
+		/* if XMS is not enabled, EMM386 emulation is impossible.
+		 * Real MS-DOS EMM386.EXE will flat out refuse to load if HIMEM.SYS is not loaded.
+		 * that also prevents VCPI from working. */
+		if (!XMS_Active() && ems_type != EMS_BOARD) {
+			if (ems_type == EMS_MIXED) {
+				LOG_MSG("EMS changed to board mode and VCPI disabled, because XMS is not enabled.");
+				ems_type = EMS_BOARD;
+			}
+			else if (ems_type == EMS_EMM386) {
+				/* do as MS-DOS does */
+				setup_EMS_none();
+				ems_type = EMS_NONE;
+				LOG_MSG("EMS disabled, EMM386 emulation is impossible when XMS is not enabled");
+				return;
+			}
 
 			ENABLE_V86_STARTUP = false;
-            ENABLE_VCPI = false;
-        }
+			ENABLE_VCPI = false;
+		}
 
-        if (ems_type != EMS_BOARD)
-            BIOS_ZeroExtendedSize(true);
+		if (ems_type != EMS_BOARD)
+			BIOS_ZeroExtendedSize(true);
 
 		dbg_zero_on_ems_allocmem = section->Get_bool("zero memory on ems memory allocation");
 		if (dbg_zero_on_ems_allocmem) {
@@ -1701,6 +1701,20 @@ public:
 			ENABLE_V86_STARTUP=false;
 		}
 
+		/* TODO: Even in "board" mode EMS emulation is highly dependent on having extended memory to map!
+		 *       If we truly wanted to emulate an EMS board, we'd allocate our own private memory and
+		 *       just map parts of it into the EMS page frame, which would allow emulation to work even
+		 *       if XMS is disabled and/or less than 1MB conventional memory is configured! */
+		if (ems_type == EMS_BOARD) {
+			if (MEM_TotalPages() <= 0x110) {
+				// FIXME: Board emulation should allocate it's own memory and map it into the EMS page frame like a device would,
+				//        not use extended memory!
+				LOG_MSG("EMS: EMS board mode as implemented still requires extended memory to work (we'll fix this someday).");
+				LOG_MSG("At least 1MB of memory must be configured for EMS emulation to work.");
+				LOG_MSG("Because of this limitation, EMS emulation will run as if no memory available.");
+			}
+		}
+
 		oshandle_memsize_16kb = (unsigned int)section->Get_int("ems system handle memory size");
 		/* convert KB to 16KB pages */
 		oshandle_memsize_16kb = (oshandle_memsize_16kb+15u)/16u;
@@ -1737,9 +1751,17 @@ public:
 			emm_segmentmappings[i].handle=NULL_HANDLE;
 		}
 
-		if (EMM_AllocateSystemHandle(oshandle_memsize_16kb) != EMM_NO_ERROR) { // allocate OS-dedicated handle (ems handle zero, 384kb)
-			LOG_MSG("EMS:Unable to allocate EMS system handle. disabling VCPI");
-			ENABLE_VCPI = false;
+		if (ems_type == EMS_BOARD) {
+			if (ENABLE_VCPI) {
+				LOG_MSG("EMS in board mode. disabling VCPI");
+				ENABLE_VCPI = false;
+			}
+		}
+		else {
+			if (EMM_AllocateSystemHandle(oshandle_memsize_16kb) != EMM_NO_ERROR) { // allocate OS-dedicated handle (ems handle zero, 384kb)
+				LOG_MSG("EMS:Unable to allocate EMS system handle. disabling VCPI");
+				ENABLE_VCPI = false;
+			}
 		}
 
 		if (ems_type == EMS_EMM386) {
@@ -1769,19 +1791,19 @@ public:
 			   in v86 mode, including protection fault exceptions */
 			call_v86mon.Install(&V86_Monitor,CB_IRET,"V86 Monitor");
 
-            {
-                Bitu old_a20 = XMS_GetEnabledA20();
+			{
+				Bitu old_a20 = XMS_GetEnabledA20();
 
-                XMS_EnableA20(true);
+				XMS_EnableA20(true);
 
-                mem_writeb((unsigned int)vcpi.private_area+0x2e00,(uint8_t)0xFE);       //GRP 4
-                mem_writeb((unsigned int)vcpi.private_area+0x2e01,(uint8_t)0x38);       //Extra Callback instruction
-                mem_writew((unsigned int)vcpi.private_area+0x2e02,call_v86mon.Get_callback());		//The immediate word
-                mem_writeb((unsigned int)vcpi.private_area+0x2e04,(uint8_t)0x66);
-                mem_writeb((unsigned int)vcpi.private_area+0x2e05,(uint8_t)0xCF);       //A IRETD Instruction
+				mem_writeb((unsigned int)vcpi.private_area+0x2e00,(uint8_t)0xFE);       //GRP 4
+				mem_writeb((unsigned int)vcpi.private_area+0x2e01,(uint8_t)0x38);       //Extra Callback instruction
+				mem_writew((unsigned int)vcpi.private_area+0x2e02,call_v86mon.Get_callback());		//The immediate word
+				mem_writeb((unsigned int)vcpi.private_area+0x2e04,(uint8_t)0x66);
+				mem_writeb((unsigned int)vcpi.private_area+0x2e05,(uint8_t)0xCF);       //A IRETD Instruction
 
-                XMS_EnableA20(old_a20 != 0);
-            }
+				XMS_EnableA20(old_a20 != 0);
+			}
 
 			/* DOSBox's default EMS emulation provides the EMS memory mapping but without the virtual 8086
 			 * mode. But there are DOS games and demos that assume EMM386.EXE == virtual 8086 mode and will
@@ -1897,12 +1919,22 @@ void EMS_DoShutDown() {
     }
 }
 
+extern uint16_t desired_ems_segment;
+
+void Update_Get_Desired_Segment(void);
+
 void EMS_PickPageFrame(void) {
+    if (desired_ems_segment == 0) Update_Get_Desired_Segment();
+
     /* the EMS page frame needs to move depending on IBM PC or PC-98 emulation.
      * IBM PC emulation can put the page frame at 0xE000 (as DOSBox has always done).
      * PC-98 emulation needs to move the page frame down because 0xE000 is taken by the 4th EGC bitplane.
      * J-3100 emulation needs to move the page frame down because  0xE000 is used in Kanji ROM. */
-    EMM_PAGEFRAME =      (IS_PC98_ARCH || IS_J3100) ? 0xD000 : 0xE000;
+    if (desired_ems_segment == 0xC000)
+        EMM_PAGEFRAME = desired_ems_segment;
+    else
+        EMM_PAGEFRAME = (IS_PC98_ARCH || IS_J3100) ? 0xD000 : 0xE000;
+
     EMM_PAGEFRAME4K =    ((EMM_PAGEFRAME*16)/4096);
 }
 
