@@ -119,6 +119,7 @@ struct VideoModeBlock {
 	Bitu	htotal,vtotal;
 	Bitu	hdispend,vdispend;
 	Bitu	special;
+	Bitu	pitch; /* bytes/scanline to use instead of normal calculation */
 	
 };
 extern VideoModeBlock ModeList_VGA[];
@@ -174,6 +175,7 @@ typedef struct {
 #define _S3_POW2_STRIDE			0x0040  /* Stride must be a power of 2, round up after offset calculation */
 #define _HIGH_DEFINITION        0x0040
 #define _UNUSUAL_MODE           0x0080
+#define _DO_NOT_LIST            0x0100  /* support the mode but do not list in VBE mode enumeration */
 #define _USER_DISABLED          0x4000  /* disabled (cannot set mode) but still listed in modelist */
 #define _USER_MODIFIED          0x8000  /* user modified (through VESAMOED) */
 
@@ -215,6 +217,8 @@ void INT10_ReadCharAttr(uint16_t * result,uint8_t page);
 void INT10_WriteChar(uint16_t chr,uint8_t attr,uint8_t page,uint16_t count,bool showattr);
 void INT10_WriteString(uint8_t row,uint8_t col,uint8_t flag,uint8_t attr,PhysPt string,uint16_t count,uint8_t page);
 
+bool CheckAnotherDisplayDriver();
+
 /* Graphics Stuff */
 void INT10_PutPixel(uint16_t x,uint16_t y,uint8_t page,uint8_t color);
 void INT10_GetPixel(uint16_t x,uint16_t y,uint8_t page,uint8_t * color);
