@@ -40,15 +40,18 @@
 using namespace std;
 
 int msgcodepage = 0, lastmsgcp = 0;
-bool morelen = false, inmsg = false, loadlang = false, uselangcp = false;
+bool morelen = false, inmsg = false, loadlang = false;
+bool uselangcp = false; // True if Language file is loaded via CONFIG -set langcp option. Use codepage specified in the language file 
 bool isSupportedCP(int newCP), CodePageHostToGuestUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), CodePageGuestToHostUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton), OpenGL_using(void);
 void InitFontHandle(void), ShutFontHandle(void), refreshExtChar(void), SetIME(void), runRescan(const char *str), menu_update_dynamic(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname), JFONT_Init(), DOSBox_SetSysMenu(), UpdateSDLDrawTexture(), makestdcp950table(), makeseacp951table();
 std::string langname = "", langnote = "", GetDOSBoxXPath(bool withexe=false);
-extern Bitu DOS_LoadKeyboardLayout(const char * layoutname, int32_t codepage, const char * codepagefile);
 extern int lastcp, FileDirExistUTF8(std::string &localname, const char *name), toSetCodePage(DOS_Shell *shell, int newCP, int opt);
 extern bool dos_kernel_disabled, force_conversion, showdbcs, dbcs_sbcs, enableime, tonoime, chinasea;
 extern uint16_t GetDefaultCP();
 extern const char * RunningProgram;
+Bitu DOS_ChangeKeyboardLayout(const char* layoutname, int32_t codepage);
+Bitu DOS_ChangeCodepage(int32_t codepage, const char* codepagefile);
+Bitu DOS_LoadKeyboardLayout(const char* layoutname, int32_t codepage, const char* codepagefile);
 
 #define LINE_IN_MAXLEN 2048
 
@@ -154,9 +157,7 @@ void AddMessages() {
     MSG_Add("CONTENT","Content:");
     MSG_Add("EDIT_FOR","Edit %s");
     MSG_Add("HELP_FOR","Help for %s");
-    MSG_Add("HELP_INFO", "Click the \"Help\" button below to see detailed help information.");
     MSG_Add("SELECT_VALUE", "Select property value");
-    MSG_Add("CONFIGURATION_FOR","Configuration for %s");
     MSG_Add("CONFIGURATION","Configuration");
     MSG_Add("SETTINGS","Settings");
     MSG_Add("LOGGING_OUTPUT","DOSBox-X logging output");
@@ -175,7 +176,6 @@ void AddMessages() {
     MSG_Add("NETWORK_LIST","Network interface list");
     MSG_Add("PRINTER_LIST","Printer device list");
     MSG_Add("INTRODUCTION","Introduction");
-    MSG_Add("CONFIGURE_GROUP", "Choose a settings group to configure:");
     MSG_Add("SHOW_ADVOPT", "Show advanced options");
     MSG_Add("USE_PRIMARYCONFIG", "Use primary config file");
     MSG_Add("USE_PORTABLECONFIG", "Use portable config file");
@@ -204,19 +204,24 @@ void AddMessages() {
     MSG_Add("AUTO_CYCLE_OFF","Auto cycles [off]");
 }
 
+void MSG_Init(void);
 void SetKEYBCP() {
     if (IS_PC98_ARCH || IS_JEGA_ARCH || IS_DOSV || dos_kernel_disabled || !strcmp(RunningProgram, "LOADLIN")) return;
-    if (msgcodepage == 437) {dos.loaded_codepage=0;DOS_LoadKeyboardLayout("us", 437, "auto");dos.loaded_codepage=437;}
-    else if (msgcodepage == 850) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("de", 850, "auto");dos.loaded_codepage=850;}
-    else if (msgcodepage == 857) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("tr", 857, "auto");dos.loaded_codepage=857;}
-    else if (msgcodepage == 858) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("es", 858, "auto");dos.loaded_codepage=858;}
-    else if (msgcodepage == 859) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("fr", 859, "auto");dos.loaded_codepage=859;}
-    else if (msgcodepage == 860) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("br", 860, "auto");dos.loaded_codepage=860;}
-    else if (msgcodepage == 932) {dos.loaded_codepage=437;DOS_LoadKeyboardLayout("jp", 932, "auto");dos.loaded_codepage=932;}
-    else if (msgcodepage == 936) {dos.loaded_codepage=0;DOS_LoadKeyboardLayout("us", 437, "auto");DOS_LoadKeyboardLayout("cn", 936, "auto");dos.loaded_codepage=936;}
-    else if (msgcodepage == 949) {dos.loaded_codepage=0;DOS_LoadKeyboardLayout("us", 437, "auto");DOS_LoadKeyboardLayout("ko", 949, "auto");dos.loaded_codepage=949;}
-    else if (msgcodepage == 950) {dos.loaded_codepage=0;DOS_LoadKeyboardLayout("us", 437, "auto");DOS_LoadKeyboardLayout("tw", 950, "auto");dos.loaded_codepage=950;}
-    else if (msgcodepage == 951) {dos.loaded_codepage=0;DOS_LoadKeyboardLayout("us", 437, "auto");DOS_LoadKeyboardLayout("hk", 951, "auto");dos.loaded_codepage=951;}
+    Bitu return_code;
+
+    if(msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950 || msgcodepage == 951) {
+        MSG_Init();
+        InitFontHandle();
+        JFONT_Init();
+        dos.loaded_codepage = msgcodepage;
+    }
+    else {
+        return_code = DOS_ChangeCodepage(858, "auto"); /* FIX_ME: Somehow requires to load codepage twice */
+        return_code = DOS_ChangeCodepage(msgcodepage, "auto");
+        if(return_code == KEYB_NOERROR) {
+            dos.loaded_codepage = msgcodepage;
+        }
+    }
     runRescan("-A -Q");
 }
 
@@ -254,13 +259,17 @@ FILE *testLoadLangFile(const char *fname) {
     return mfile;
 }
 
+char loaded_fname[LINE_IN_MAXLEN + 1024];
 void LoadMessageFile(const char * fname) {
 	if (!fname) return;
 	if(*fname=='\0') return;//empty string=no languagefile
+    if (!strcmp(fname, loaded_fname)){
+        //LOG_MSG("Message file %s already loaded.",fname);
+        return;
+    }
+    strcpy(loaded_fname, fname);
+	LOG(LOG_MISC,LOG_DEBUG)("Loading message file %s",loaded_fname);
 
-	LOG(LOG_MISC,LOG_DEBUG)("Loading message file %s",fname);
-
-	lastmsgcp = 0;
 	FILE * mfile=testLoadLangFile(fname);
 	/* This should never happen and since other modules depend on this use a normal printf */
 	if (!mfile) {
@@ -271,7 +280,6 @@ void LoadMessageFile(const char * fname) {
 		control->opt_lang = "";
 		return;
 	}
-    msgcodepage = 0;
     langname = langnote = "";
 	char linein[LINE_IN_MAXLEN+1024];
 	char name[LINE_IN_MAXLEN+1024], menu_name[LINE_IN_MAXLEN], mapper_name[LINE_IN_MAXLEN];
@@ -280,9 +288,10 @@ void LoadMessageFile(const char * fname) {
 	name[0]=0;string[0]=0;
 	morelen=inmsg=true;
     bool res=true;
+    bool loadlangcp = false;
     int cp=dos.loaded_codepage;
     if (!dos.loaded_codepage) res=InitCodePage();
-	while(fgets(linein, LINE_IN_MAXLEN+1024, mfile)!=0) {
+	while(fgets(linein, LINE_IN_MAXLEN+1024, mfile)) {
 		/* Parse the read line */
 		/* First remove characters 10 and 13 from the line */
 		char * parser=linein;
@@ -295,7 +304,6 @@ void LoadMessageFile(const char * fname) {
 			parser++;
 		}
 		*writer=0;
-
 		/* New string name */
 		if (linein[0]==':') {
             string[0]=0;
@@ -306,20 +314,26 @@ void LoadMessageFile(const char * fname) {
                     *r=0;
                     if (!strcmp(p, "CODEPAGE")) {
                         int c = atoi(r+1);
-                        if ((!res || control->opt_langcp || uselangcp) && c>0 && isSupportedCP(c)) {
-                            if (((IS_PC98_ARCH||IS_JEGA_ARCH) && c!=437 && c!=932 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the Japanese PC-98 or JEGA/AX system.\n\nAre you sure to use the language file for this machine type?", "yesno","question", 2)) || (((IS_JDOSV && c!=932) || (IS_PDOSV && c!=936) || (IS_KDOSV && c!=949) || (IS_TDOSV && c!=950 && c!=951)) && c!=437 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the current DOS/V system.\n\nAre you sure to use the language file for this system type?", "yesno","question", 2))) {
+                        if(!isSupportedCP(c)) {
+                            LOG_MSG("Language file: Invalid codepage :DOSBOX-X:CODEPAGE:%d", c);
+                            loadlangcp = false;
+                        }
+                        else if (((IS_PC98_ARCH||IS_JEGA_ARCH) && c!=437 && c!=932 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the Japanese PC-98 or JEGA/AX system.\n\nAre you sure to use the language file for this machine type?", "yesno","question", 2)) || (((IS_JDOSV && c!=932) || (IS_PDOSV && c!=936) || (IS_KDOSV && c!=949) || (IS_TDOSV && c!=950 && c!=951)) && c!=437 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the current DOS/V system.\n\nAre you sure to use the language file for this system type?", "yesno","question", 2))) {
                                 fclose(mfile);
                                 dos.loaded_codepage = cp;
                                 return;
-                            }
-                            std::string msg = "The specified language file uses code page " + std::to_string(c) + ". Do you want to change to this code page accordingly?";
-                            if (!control->opt_langcp && !uselangcp && c != 437 && GetDefaultCP() == 437 && systemmessagebox("DOSBox-X language file", msg.c_str(), "yesno", "question", 1)) control->opt_langcp = true;
-                            msgcodepage = c;
-                            dos.loaded_codepage = c;
-                            if (c == 950 && !chinasea) makestdcp950table();
-                            if (c == 951 && chinasea) makeseacp951table();
                         }
-                        lastmsgcp = c;
+                        else {
+                            std::string msg = "The specified language file uses code page " + std::to_string(c) + ". Do you want to change to this code page accordingly?";
+                            if(c != dos.loaded_codepage && (control->opt_langcp || uselangcp || !loadlang || (loadlang && systemmessagebox("DOSBox-X language file", msg.c_str(), "yesno", "question", 1)))){
+                                loadlangcp = true;
+                                msgcodepage = c;
+                                dos.loaded_codepage = c;
+                                if (c == 950 && !chinasea) makestdcp950table();
+                                if (c == 951 && chinasea) makeseacp951table();
+                                lastmsgcp = c;
+                            }
+                        }
                     } else if (!strcmp(p, "LANGUAGE"))
                         langname = r+1;
                     else if (!strcmp(p, "REMARK"))
@@ -373,15 +387,14 @@ void LoadMessageFile(const char * fname) {
     menu_update_autocycle();
     update_bindbutton_text();
     dos.loaded_codepage=cp;
-    if ((control->opt_langcp || uselangcp) && msgcodepage>0 && isSupportedCP(msgcodepage) && msgcodepage != dos.loaded_codepage) {
+    if (loadlangcp && msgcodepage>0 && isSupportedCP(msgcodepage) && msgcodepage != dos.loaded_codepage) {
         ShutFontHandle();
-        if (msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950 || msgcodepage == 951) {
+        if(msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950 || msgcodepage == 951) {
             dos.loaded_codepage = msgcodepage;
             InitFontHandle();
             JFONT_Init();
-            dos.loaded_codepage = cp;
         }
-        if (uselangcp && !IS_DOSV && !IS_JEGA_ARCH) {
+        if (!IS_DOSV && !IS_JEGA_ARCH) {
 #if defined(USE_TTF)
             if (ttf.inUse) toSetCodePage(NULL, msgcodepage, -2); else
 #endif
@@ -396,10 +409,9 @@ void LoadMessageFile(const char * fname) {
             SetKEYBCP();
         }
     }
-
     refreshExtChar();
-    LOG_MSG("Loaded language file: %s",fname);
-	loadlang=true;
+    LOG_MSG("LoadMessageFile: Loaded language file: %s",fname);
+    loadlang = true;
 }
 
 const char * MSG_Get(char const * msg) {
@@ -483,7 +495,7 @@ void MSG_Init() {
         }
         if (tonoime) {
             tonoime = enableime = false;
-#if defined(WIN32) && !defined(HX_DOS)
+#if defined(WIN32) && !defined(HX_DOS) && !defined(_WIN32_WINDOWS)
             ImmDisableIME((DWORD)(-1));
 #endif
             SetIME();
