@@ -962,7 +962,6 @@ void DOS_Shell::InputCommand(char * line) {
                         ++w_count;
                         if (w_count % col == 0) {p_count++;WriteOut_NoParsing("\n");lastcr=true;}
                     }
-                    size_t GetPauseCount();
                     if (p_count>GetPauseCount()) {
                         WriteOut(MSG_Get("SHELL_CMD_PAUSE"));
                         lastcr=false;
@@ -1267,7 +1266,9 @@ bool DOS_Shell::Execute(char* name, const char* args) {
 	/* check for a drive change */
 	if (((strcmp(name + 1, ":") == 0) || (strcmp(name + 1, ":\\") == 0)) && isalpha(*name) && !control->SecureMode())
 	{
+#ifdef WIN32
 		uint8_t c;uint16_t n;
+#endif
 		if (strrchr_dbcs(name,'\\')) { WriteOut(MSG_Get("SHELL_EXECUTE_ILLEGAL_COMMAND"),name); return true; }
 		if (!DOS_SetDrive(toupper(name[0])-'A')) {
 #ifdef WIN32
@@ -1359,8 +1360,7 @@ continue_1:
 
 	/*always disallow files without extension from being executed. */
 	/*only internal commands can be run this way and they never get in this handler */
-	if(extension == 0)
-	{
+	if (!extension) {
 		//Check if the result will fit in the parameters. Else abort
 		if(strlen(fullname) >( DOS_PATHLENGTH - 1) ) return false;
         char temp_name[DOS_PATHLENGTH + 4];
@@ -1572,7 +1572,7 @@ bool DOS_Shell::hasExecutableExtension(const char* name) {
 
 char * DOS_Shell::Which(char * name) {
 	size_t name_len = strlen(name);
-	if(name_len >= DOS_PATHLENGTH) return 0;
+	if(name_len >= DOS_PATHLENGTH) return nullptr;
 
 	/* Parse through the Path to find the correct entry */
 	/* Check if name is already ok but just misses an extension */
@@ -1603,11 +1603,11 @@ char * DOS_Shell::Which(char * name) {
 
 	/* No Path in filename look through path environment string */
 	char path[DOS_PATHLENGTH];std::string temp;
-	if (!GetEnvStr("PATH",temp)) return 0;
+	if (!GetEnvStr("PATH",temp)) return nullptr;
 	const char * pathenv=temp.c_str();
-	if (!pathenv) return 0;
+	if (!pathenv) return nullptr;
 	pathenv = strchr(pathenv,'=');
-	if (!pathenv) return 0;
+	if (!pathenv) return nullptr;
 	pathenv++;
 	while (*pathenv) {
 		/* remove ; and ;; at the beginning. (and from the second entry etc) */
@@ -1671,5 +1671,5 @@ char * DOS_Shell::Which(char * name) {
 			}
 		}
 	}
-	return 0;
+	return nullptr;
 }
